@@ -436,6 +436,48 @@ void test_array_generate_empty_strings()
     "Array of 12 empty strings came out wrong.");
 }
 
+void test_sparse_arrays()
+{
+  // Reproduce #922 : NULL not paying for its separator in an array, causing
+  // problems in sparse arrays.
+
+  // If NULL didn't pay for its separator, the size allocated for an array-like
+  // object filled with null-like values would be too small.
+
+  auto arrayOfNulls = std::vector<std::optional<int>>(4, std::nullopt);
+  std::string arrayOfNullsStr = "{NULL,NULL,NULL,NULL}";
+
+  PQXX_CHECK(
+    pqxx::size_buffer(arrayOfNulls) >= arrayOfNullsStr.size(),
+    "Buffer size allocated for an array of optional<int> filled with nulls "
+    "was too small.");
+
+  PQXX_CHECK_EQUAL(
+    pqxx::to_string(arrayOfNulls), arrayOfNullsStr,
+    "Array of optional<int> filled with std::nullopt came out wrong. ");
+
+
+  // If the array-like object is instead sparsely-filled, the values contained
+  // in non-null elements may leave behind excess unused size which hides the
+  // problem better - it only becomes an error once the array contains enough
+  // null-like values to outweigh this excess unused size.
+
+  std::array<std::optional<int>, 14> sparseArray;
+  sparseArray[sparseArray.size() - 1] = 42;
+
+  std::string sparseArrayStr =
+    "{NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,NULL,"
+    "42}";
+
+  PQXX_CHECK(
+    pqxx::size_buffer(sparseArray) >= sparseArrayStr.size(),
+    "Buffer size allocated for a sparsely-filled array of optional<int> was "
+    "too small.");
+
+  PQXX_CHECK_EQUAL(
+    pqxx::to_string(sparseArray), sparseArrayStr,
+    "A sparsely-filled array of optional<int> came out wrong.");
+}
 
 void test_array_generate()
 {
@@ -445,17 +487,19 @@ void test_array_generate()
   test_generate_multiple_items();
   test_generate_nested_array();
   test_generate_escaped_strings();
+  test_sparse_arrays();
 }
 
 
 void test_array_roundtrip()
 {
-  pqxx::connection c;
-  pqxx::work w{c};
+  pqxx::connection cx;
+  pqxx::work tx{cx};
 
   std::vector<int> const in{0, 1, 2, 3, 5};
-  auto const r1{w.exec1("SELECT " + c.quote(in) + "::integer[]")};
-  pqxx::array_parser parser{r1[0].view()};
+  auto const text{
+    tx.query_value<std::string>("SELECT $1::integer[]", pqxx::params{in})};
+  pqxx::array_parser parser{text};
   auto item{parser.get_next()};
   PQXX_CHECK_EQUAL(
     item.first, pqxx::array_parser::juncture::row_start,
@@ -496,8 +540,8 @@ void test_array_strings()
 
   for (auto const &input : inputs)
   {
-    auto const r{tx.exec_params("SELECT ARRAY[$1]", input)};
-    pqxx::array_parser parser{r[0][0].as<std::string_view>()};
+    auto const f{tx.exec("SELECT ARRAY[$1]", pqxx::params{input}).one_field()};
+    pqxx::array_parser parser{f.as<std::string_view>()};
     auto [start_juncture, start_value]{parser.get_next()};
     pqxx::ignore_unused(start_value);
     PQXX_CHECK_EQUAL(
@@ -699,7 +743,7 @@ void test_as_sql_array()
   pqxx::row r;
   {
     pqxx::work tx{cx};
-    r = tx.exec1("SELECT ARRAY [5, 4, 3, 2]");
+    r = tx.exec("SELECT ARRAY [5, 4, 3, 2]").one_row();
     // Connection closes, but we should still be able to parse the array.
   }
   auto const array{r[0].as_sql_array<int>()};
